@@ -8,7 +8,10 @@ This module creates the argparse Parser.
 
 from __future__ import annotations
 import sys
-import argparse, json
+import argparse
+import json
+import re
+from datetime import datetime, timedelta, timezone
 from meerschaum.utils.typing import Union, Dict, List, Any, Tuple, Callable
 from meerschaum.utils.misc import string_to_dict
 from meerschaum.utils.warnings import error
@@ -35,33 +38,91 @@ class ArgumentParser(argparse.ArgumentParser):
         return result
 
 
-def parse_datetime(dt_str: str) -> datetime.datetime:
+def parse_datetime(dt_str: str) -> Union[datetime, int, str]:
     """Parse a string into a datetime."""
+    from meerschaum.utils.dtypes import round_time
     from meerschaum.utils.misc import is_int
     if is_int(dt_str):
         return int(dt_str)
 
+    if dt_str in ('None', '[None]'):
+        return 'None'
+
     from meerschaum.utils.packages import attempt_import
-    import datetime
-    dateutil_parser = attempt_import('dateutil.parser')
+    dateutil_parser, dateutil_relativedelta = attempt_import(
+        'dateutil.parser', 'dateutil.relativedelta'
+    )
+    relativedelta = dateutil_relativedelta.relativedelta
+
+    ago_pattern = r'(\d+)\s+(days?|minutes?|seconds?|hours?|weeks?|months?|years?)\s+ago'
+    round_pattern = r'(\d+)\s+(days?|minutes?|seconds?|hours?|weeks?|years?)'
+    now = datetime.now(timezone.utc).replace(tzinfo=None)
+    ago_matches = re.findall(ago_pattern, dt_str.lower())
 
     try:
         if dt_str.lower() == 'now':
-            dt = datetime.datetime.utcnow()
+            dt = now
+        elif ago_matches:
+            val_str, unit_str = ago_matches[0]
+            if not unit_str.endswith('s'):
+                unit_str += 's'
+            val = int(val_str) if is_int(val_str) else float(val_str)
+            ago_delta = relativedelta(**{unit_str: val})
+            round_part = dt_str.lower().split('ago ')[-1]
+            round_delta = None
+            if round_part:
+                round_matches = re.findall(round_pattern, round_part)
+                if round_matches:
+                    round_val_str, round_unit_str = round_matches[0]
+                    if not round_unit_str.endswith('s'):
+                        round_unit_str += 's'
+                    round_val = (
+                        int(round_val_str)
+                        if is_int(round_val_str)
+                        else float(round_val_str)
+                    )
+                    round_delta = timedelta(**{round_unit_str: round_val})
+
+            dt = now - ago_delta
+            if round_delta is not None:
+                dt = round_time(dt, round_delta)
         else:
             dt = dateutil_parser.parse(dt_str)
-    except Exception as e:
+    except Exception:
         dt = None
     if dt is None:
-        from meerschaum.utils.warnings import warn, error
+        from meerschaum.utils.warnings import error
         error(f"'{dt_str}' is not a valid datetime format.", stack=False)
+
+    if isinstance(dt, datetime):
+        from meerschaum.utils.dtypes import coerce_timezone
+        dt = coerce_timezone(dt)
+
     return dt
 
-def parse_help(sysargs : Union[List[str], Dict[str, Any]]) -> None:
+
+def parse_executor_keys(executor_keys_str: str) -> Union[str, None]:
+    """
+    Ensure that only API keys are provided for executor_keys.
+    """
+    if executor_keys_str in ('local', 'systemd'):
+        return executor_keys_str
+
+    if executor_keys_str.lower() == 'none':
+        return 'local'
+
+    if not executor_keys_str.startswith('api:'):
+        from meerschaum.utils.warnings import error
+        error(f"Invalid exectutor keys '{executor_keys_str}'.", stack=False)
+
+    return executor_keys_str
+
+
+def parse_help(sysargs: Union[List[str], Dict[str, Any]]) -> None:
     """Parse the `--help` flag to determine which help message to print."""
     from meerschaum._internal.arguments._parse_arguments import parse_arguments, parse_line
     from meerschaum.actions import actions, get_subactions
-    import importlib, inspect, textwrap
+    import textwrap
     from copy import deepcopy
     if isinstance(sysargs, list):
         args = parse_arguments(sysargs)
@@ -79,21 +140,26 @@ def parse_help(sysargs : Union[List[str], Dict[str, Any]]) -> None:
     ### Check for subactions.
     if len(args['action']) > 1:
         try:
-            subaction = get_subactions(args['action'][0])[args['action'][1]]
-        except Exception as e:
+            subactions = get_subactions(args['action'][0])
+            subaction_name = args['action'][1]
+            if subaction_name not in subactions:
+                subaction_name = subaction_name + 's'
+            subaction = subactions[subaction_name]
+        except Exception:
             subaction = None
         if subaction is not None:
             return print(textwrap.dedent(subaction.__doc__))
 
     try:
         doc = actions[args['action'][0]].__doc__
-    except Exception as e:
+    except Exception:
         doc = None
     if doc is None:
         doc = "No help available for '" + f"{args['action'][0]}" + "'."
     return print(textwrap.dedent(doc))
 
-def parse_version(sysargs : List[str]):
+
+def parse_version(sysargs: List[str]):
     """Print the Meerschaum version."""
     from meerschaum.config import __version__ as version
     from meerschaum.config import __doc__ as doc
@@ -101,6 +167,15 @@ def parse_version(sysargs : List[str]):
         return print(version)
     return print(doc)
 
+
+def parse_name(name_str: str) -> Union[str, None]:
+    """
+    Ensure that `--name` is not an empty string.
+    """
+    if not name_str:
+        return None
+    return name_str
+
 def get_arguments_triggers() -> Dict[str, Tuple[str]]:
     """ """
     triggers = {}
@@ -120,10 +195,12 @@ _seen_plugin_args = {}
 
 groups = {}
 groups['actions'] = parser.add_argument_group(title='Actions options')
+groups['jobs'] = parser.add_argument_group(title='Jobs options')
 groups['pipes'] = parser.add_argument_group(title='Pipes options')
 groups['sync'] = parser.add_argument_group(title='Sync options')
 groups['api'] = parser.add_argument_group(title='API options')
 groups['plugins'] = parser.add_argument_group(title='Plugins options')
+groups['tokens'] = parser.add_argument_group(title='Tokens options')
 groups['packages'] = parser.add_argument_group(title='Packages options')
 groups['debug'] = parser.add_argument_group(title='Debugging options')
 groups['misc'] = parser.add_argument_group(title='Miscellaneous options')
@@ -147,34 +224,9 @@ groups['actions'].add_argument(
     '-f', '--force', action="store_true", help="Override safety checks"
 )
 groups['actions'].add_argument(
-    '--noask', action='store_true',
+    '--noask', '--no-ask', action='store_true',
     help="Automatically choose the defaults answers to questions. Does not result in data loss.",
 )
-groups['actions'].add_argument(
-    '-d', '--daemon', action='store_true',
-    help = "Run an action as a background daemon."
-)
-#  groups['actions'].add_argument(
-    #  '--keep-logs', '--keep-job', '--save-job', '--keep-daemon-output', '--skip-daemon-cleanup',
-    #  action='store_true', help="Preserve a job's output files for later inspection."
-#  )
-
-groups['actions'].add_argument(
-    '--rm', action='store_true', help="Delete a job once it has finished executing."
-)
-groups['actions'].add_argument(
-    '--name', '--job-name', type=str, help=(
-        "Assign a name to a job. If no name is provided, a random name will be assigned."
-    ),
-)
-groups['actions'].add_argument(
-    '-s', '--schedule', '--cron', type=str,
-    help = (
-        "Continue executing the action according to a schedule (e.g. 'every 1 seconds'). \n"
-        + "Often used with `-d`. See this page for scheduling syntax:\n "
-        + "https://red-engine.readthedocs.io/en/stable/condition_syntax/index.html"
-    )
-)
 groups['actions'].add_argument(
     '-A', '--sub-args', nargs=argparse.REMAINDER,
     help = (
@@ -184,6 +236,38 @@ groups['actions'].add_argument(
     )
 )
 
+### Jobs options
+groups['jobs'].add_argument(
+    '-d', '--daemon', action='store_true',
+    help = "Run an action as a background daemon to create a job."
+)
+groups['jobs'].add_argument(
+    '--name', '--job-name', type=parse_name, help=(
+        "Assign a name to a job. If no name is provided, a random name will be assigned."
+    ),
+)
+groups['jobs'].add_argument(
+    '-s', '--schedule', '--cron', type=str,
+    help = (
+        "Continue executing the action according to a schedule (e.g. 'every 1 seconds'). \n"
+        + "Often used with `-d`. See this page for scheduling syntax:\n "
+        + "https://red-engine.readthedocs.io/en/stable/condition_syntax/index.html"
+    )
+)
+groups['jobs'].add_argument(
+    '--restart', action='store_true',
+    help=("Restart a job if not stopped manually."),
+)
+groups['jobs'].add_argument(
+    '-e', '--executor-keys', type=parse_executor_keys,
+    help=(
+        "Execute jobs locally or remotely. "
+        "Supported values are 'local', 'systemd', and 'api:{label}'."
+    ),
+)
+groups['jobs'].add_argument(
+    '--rm', action='store_true', help="Delete a job once it has finished executing."
+)
 ### Pipes options
 groups['pipes'].add_argument(
     '-c', '-C', '--connector-keys', nargs='+',
@@ -224,28 +308,80 @@ groups['sync'].add_argument(
     help="Run the action asynchronously, if possible. Alias for --unblock",
 )
 groups['sync'].add_argument(
-    '--begin', type=parse_datetime, help="Specify a begin datetime for syncing or displaying data."
+    '--begin', type=parse_datetime, help="The begin datetime when syncing, fetching data."
 )
 groups['sync'].add_argument(
-    '--end', type=parse_datetime, help="Specify an end datetime for syncing or displaying data."
+    '--end', type=parse_datetime, help="The end datetime when syncing, fetching data."
 )
 groups['sync'].add_argument(
-    '--chunksize', type=int, help=(
-        "Specify the chunksize for syncing and retrieving data. Defaults to 900."
+    '--chunksize', type=int, help="How many rows per chunk. Defaults to 100,000."
+)
+groups['sync'].add_argument(
+    '--batchsize', type=int, help=(
+        "How many chunks to process in parallel. Defaults to number of CPUs."
     ),
 )
+groups['sync'].add_argument(
+    '--chunk-minutes', type=int, help=(
+        "Specify the chunk interval in minutes for verification syncs. Defaults to 1440."
+    ),
+)
+groups['sync'].add_argument(
+    '--chunk-hours', type=int, help=(
+        "Specify the chunk interval in days for verification syncs. See `--chunk-minutes`."
+    ),
+)
+groups['sync'].add_argument(
+    '--chunk-days', type=int, help=(
+        "Specify the chunk interval in days for verification syncs. See `--chunk-minutes`."
+    ),
+)
+groups['sync'].add_argument(
+    '--verify', action="store_true", help="Completely resync a pipe's entire interval.",
+)
+groups['sync'].add_argument(
+    '--deduplicate', '--dedup', action="store_true",
+    help="Remove duplicate rows from a pipe's table.",
+)
+groups['sync'].add_argument(
+    '--bounded', '--bound', action="store_true",
+    help=(
+        "When verifying, do not sync outside "
+        "the existing oldest and newest datetime bounds."
+    )
+)
 groups['sync'].add_argument(
     '--skip-check-existing', '--allow-duplicates', action='store_true',
-    help = (
-        "Skip checking for duplicate rows when syncing. " +
-        "This drastically improves performance when all rows to be synced are unique. " +
-        "For example, this setting is highly recommended for use with IoT devices."
+    help=(
+        "Skip checking for duplicate rows when syncing. "
+        "This improves performance when all rows to be synced are unique. "
+    )
+)
+groups['sync'].add_argument(
+    '--skip-enforce-dtypes', action='store_true',
+    help=(
+        "Skip enforcing incoming data to a pipe's dtypes. "
+        "This improves performance when all rows are expected to already be of the correct type."
+    ),
+)
+groups['sync'].add_argument(
+    '--skip-chunks-with-greater-rowcounts', action='store_true',
+    help="When verifying, skip chunks with rowcounts greater than the remote's."
+)
+groups['sync'].add_argument(
+    '--check-rowcounts-only', action='store_true', help=(
+        "Only compare row-counts when verifying pipes."
+    ),
+)
+groups['sync'].add_argument(
+    '--skip-hooks', action='store_true', help=(
+        "Skip calling the sync hooks."
     )
 )
 groups['sync'].add_argument(
     '--cache', action='store_true',
-    help = (
-        "When syncing or viewing a pipe's data, sync to a local database for later analysis."
+    help=(
+        "Sync pipes' metadata to disk."
     )
 )
 
@@ -257,26 +393,46 @@ groups['api'].add_argument(
     '--host', type=str,
     help="The host address to bind to for the API server. Defaults to '0.0.0.0'."
 )
+groups['api'].add_argument(
+    '--webterm-port', type=int,
+    help="The port on which to run the webterm server.",
+)
 groups['api'].add_argument(
     '-w', '--workers', type=int,
     help = "How many workers to run a concurrent action (e.g. running the API or syncing pipes)"
 )
 groups['api'].add_argument(
     '--no-dash', '--nodash', action='store_true',
-    help = 'When starting the API, do not start the Web interface.',
+    help = 'When starting the API, do not start the Web Console dashboard.',
+)
+groups['api'].add_argument(
+    '--no-webterm', '--nowebterm', action='store_true',
+    help="When starting the API, do not start the Webterm.",
 )
 groups['api'].add_argument(
     '--private', '--private-mode', action='store_true',
     help = 'Require authentication for all endpoints.',
 )
 groups['api'].add_argument(
-    '--no-auth', '--noauth', action='store_true',
+    '--secure', action='store_true',
+    help = "Start the API in secure mode, only allowing `admin` users to execute actions.",
+)
+groups['api'].add_argument(
+    '--no-auth', '--noauth', '--insecure', action='store_true',
     help = 'When starting the API, do not require authentication. WARNING: This is dangerous!',
 )
 groups['api'].add_argument(
     '--production', '--gunicorn', action='store_true',
     help = 'Start the API with the Gunicorn process manager.'
 )
+groups['api'].add_argument(
+    '--keyfile', type=str,
+    help = "Start the API server with this keyfile (requires --certfile).",
+)
+groups['api'].add_argument(
+    '--certfile', type=str,
+    help = "Start the API server with this certfile (requires --keyfile).",
+)
 
 ### Plugins options
 groups['plugins'].add_argument(
@@ -284,6 +440,16 @@ groups['plugins'].add_argument(
     help="Meerschaum plugins repository to connect to. Specify an API label (default: 'mrsm')"
 )
 
+### Tokens options
+groups['tokens'].add_argument(
+    '--ttl-days', type=int,
+    help="For how many days should a token be valid.",
+)
+groups['tokens'].add_argument(
+    '--scopes', '--scope', nargs='+',
+    help="Which scope permissions to grant to a token.",
+)
+
 ### Packages options
 groups['packages'].add_argument(
     '--venv', type=str,
@@ -309,6 +475,9 @@ groups['misc'].add_argument(
 groups['misc'].add_argument(
     '--nopretty', action="store_true", help="Print elements without 'pretty' formatting"
 )
+groups['misc'].add_argument(
+    '--skip-deps', action="store_true", help="Skip dependencies when installing plugins.",
+)
 groups['misc'].add_argument(
     '-P', '--params', type=string_to_dict, help=(
         "Parameters dictionary in JSON format or simple format. " +
@@ -347,3 +516,7 @@ groups['misc'].add_argument(
         + "Default behavior is to only allow recognized actions."
     )
 )
+groups['misc'].add_argument(
+    '--no-daemon', action='store_true',
+    help="Do not use the Meerschaum CLI daemon to execute the action.",
+)
