@@ -10,13 +10,13 @@ from __future__ import annotations
 from meerschaum.utils.typing import List, Any, SuccessTuple, Optional, Union
 
 def install(
-        action: Optional[List[str]] = None,
-        **kw: Any
-    ) -> SuccessTuple:
+    action: Optional[List[str]] = None,
+    **kw: Any
+) -> SuccessTuple:
     """
     Install Meerschaum plugins or Python packages.
     """
-    from meerschaum.utils.misc import choose_subaction
+    from meerschaum.actions import choose_subaction
     options = {
         'plugins'  : _install_plugins,
         'packages' : _install_packages,
@@ -26,9 +26,9 @@ def install(
 
 
 def _complete_install(
-        action: Optional[List[str]] = None,
-        **kw: Any
-    ) -> List[str]:
+    action: Optional[List[str]] = None,
+    **kw: Any
+) -> List[str]:
     """
     Override the default Meerschaum `complete_` function.
     """
@@ -52,12 +52,13 @@ def _complete_install(
 
 
 def _install_plugins(
-        action: Optional[List[str]] = None,
-        repository: Optional[str] = None,
-        force: bool = False,
-        debug: bool = False,
-        **kw: Any
-    ) -> SuccessTuple:
+    action: Optional[List[str]] = None,
+    repository: Optional[str] = None,
+    skip_deps: bool = False,
+    force: bool = False,
+    debug: bool = False,
+    **kw: Any
+) -> SuccessTuple:
     """
     Install a plugin.
     
@@ -73,36 +74,34 @@ def _install_plugins(
         - install plugins noaa --repo mrsm  (mrsm is the default instance)
         - install plugins noaa --repo mycustominstance
     """
-    from meerschaum.utils.debug import dprint
     from meerschaum.utils.warnings import info
-    from meerschaum.plugins import reload_plugins
     from meerschaum.connectors.parse import parse_repo_keys
-    import meerschaum.actions
+    from meerschaum.actions import actions
     from meerschaum.utils.formatting import print_tuple
-    from meerschaum.core import Plugin
-    from meerschaum.connectors.api import APIConnector
 
     if not action:
         return False, "No plugins to install."
 
     repo_connector = parse_repo_keys(repository)
 
-    successes = {}
     for name in action:
         info(f"Installing plugin '{name}' from Meerschaum repository '{repo_connector}'...")
-        success, msg = repo_connector.install_plugin(name, force=force, debug=debug)
-        successes[name] = (success, msg)
+        success, msg = repo_connector.install_plugin(
+            name,
+            force = force,
+            skip_deps = skip_deps,
+            debug = debug,
+        )
         print_tuple((success, msg))
 
-    reload_plugins(debug=debug)
-    return True, "Success"
+    return actions['reload'](debug=debug)
 
 
 def _complete_install_plugins(
-        action: Optional[List[str]] = None,
-        repository: Optional[str] = None,
-        **kw: Any
-    ) -> List[str]:
+    action: Optional[List[str]] = None,
+    repository: Optional[str] = None,
+    **kw: Any
+) -> List[str]:
     """
     Search for plugins to autocomplete command line text.
     NOTE: Disabled for the time being so we don't interrupt the user typing.
@@ -128,8 +127,9 @@ def _complete_install_plugins(
     from meerschaum.connectors.parse import parse_repo_keys
     try:
         repo_connector = parse_repo_keys(repository)
-    except Exception as e:
+    except Exception:
         return []
+
     results = repo_connector.get_plugins(search_term=search_term)
     if len(results) == 1 and results[0] == search_term:
         return []
@@ -139,12 +139,12 @@ class NoVenv:
     pass
 
 def _install_packages(
-        action: Optional[List[str]] = None,
-        sub_args: Optional[List[str]] = None,
-        venv: Union[str, NoVenv] = NoVenv,
-        debug: bool = False,
-        **kw: Any
-    ) -> SuccessTuple:
+    action: Optional[List[str]] = None,
+    sub_args: Optional[List[str]] = None,
+    venv: Union[str, NoVenv] = NoVenv,
+    debug: bool = False,
+    **kw: Any
+) -> SuccessTuple:
     """
     Install PyPI packages into the Meerschaum virtual environment.
     
@@ -152,10 +152,11 @@ def _install_packages(
         `install packages pandas numpy`
     """
     if not action:
-        return False, f"No packages to install."
-    from meerschaum.utils.warnings import info
+        return False, "No packages to install."
+
     from meerschaum.utils.packages import pip_install
     from meerschaum.utils.misc import items_str
+    from meerschaum.actions import actions
     if venv is NoVenv:
         venv = 'mrsm'
 
@@ -165,29 +166,32 @@ def _install_packages(
         venv = venv,
         debug = debug,
     ):
+        reload_success, reload_msg = actions['reload'](debug=debug)
+        if not reload_success:
+            return reload_success, reload_msg
+
         return True, (
             "Successfully installed package" + ("s" if len(action) != 1 else '')
             + f" {items_str(action)}"
             + f" into the virtual environment '{venv}'."
         )
+
     return False, (
-        f"Failed to install package" + ("s" if len(action) != 1 else '') + f" {items_str(action)}."
+        "Failed to install package" + ("s" if len(action) != 1 else '') + f" {items_str(action)}."
     )
 
 
-def _complete_install_packages(
-        **kw : Any
-    ) -> List[str]:
+def _complete_install_packages(**kw : Any) -> List[str]:
     return []
 
 
 def _install_required(
-        action: Optional[List[str]] = None,
-        repository: Optional[str] = None,
-        force: bool = False,
-        debug: bool = False,
-        **kw
-    ) -> SuccessTuple:
+    action: Optional[List[str]] = None,
+    repository: Optional[str] = None,
+    force: bool = False,
+    debug: bool = False,
+    **kw
+) -> SuccessTuple:
     """
     Install the required packages for Meerschaum plugins.
     Each plugin's packages will be installed into its virtual environment.
@@ -198,8 +202,8 @@ def _install_required(
     from meerschaum.core import Plugin
     from meerschaum.utils.warnings import warn, info
     from meerschaum.connectors.parse import parse_repo_keys
-    from meerschaum.utils.formatting import print_tuple
     from meerschaum.plugins import get_plugins_names
+    from meerschaum.actions import actions
     repo_connector = parse_repo_keys(repository)
 
     plugins_names = action or get_plugins_names()
@@ -220,7 +224,11 @@ def _install_required(
         else:
             success_count += 1
 
-    success = success_count > 0
+    reload_success, reload_msg = actions['reload'](debug=debug)
+    if not reload_success:
+        return reload_success, reload_msg
+
+    success = fail_count == 0
     msg = (
         f"Installed packages for {success_count + fail_count} plugins\n    "
         + f"({success_count} succeeded, {fail_count} failed)."
@@ -233,8 +241,9 @@ def _complete_install_required(*args, **kw) -> List[str]:
     return _complete_uninstall_plugins(*args, **kw)
 
 
+
 ### NOTE: This must be the final statement of the module.
 ###       Any subactions added below these lines will not
 ###       be added to the `help` docstring.
-from meerschaum.utils.misc import choices_docstring as _choices_docstring
+from meerschaum.actions import choices_docstring as _choices_docstring
 install.__doc__ += _choices_docstring('install')
