@@ -9,21 +9,22 @@ Upgrade your current Meerschaum environment
 from __future__ import annotations
 from meerschaum.utils.typing import SuccessTuple, Any, List, Optional, Union
 
+
 def upgrade(
-        action: Optional[List[str]] = None,
-        **kw: Any
-    ) -> SuccessTuple:
+    action: Optional[List[str]] = None,
+    **kw: Any
+) -> SuccessTuple:
     """
     Upgrade Meerschaum, plugins, or packages.
-    
+
     Command:
         `upgrade {option}`
-    
+
     Example:
         `upgrade meerschaum`
     """
 
-    from meerschaum.utils.misc import choose_subaction
+    from meerschaum.actions import choose_subaction
     options = {
         'plugins'    : _upgrade_plugins,
         'meerschaum' : _upgrade_meerschaum,
@@ -32,14 +33,15 @@ def upgrade(
     }
     return choose_subaction(action, options, **kw)
 
+
 def _upgrade_meerschaum(
-        action: Optional[List[str]] = None,
-        yes: bool = False,
-        force: bool = False,
-        noask: bool = False,
-        debug: bool = False,
-        **kw: Any
-    ) -> SuccessTuple:
+    action: Optional[List[str]] = None,
+    yes: bool = False,
+    force: bool = False,
+    noask: bool = False,
+    debug: bool = False,
+    **kw: Any
+) -> SuccessTuple:
     """
     Upgrade the current Meerschaum instance.
     Optionally specify dependency groups.
@@ -49,19 +51,16 @@ def _upgrade_meerschaum(
         - `upgrade meerschaum full`
 
     """
-    import subprocess
-    import json
     from meerschaum.utils.debug import dprint
     from meerschaum.utils.warnings import info
     from meerschaum.actions import actions
     from meerschaum.utils.prompt import yes_no
-    from meerschaum.utils.packages import pip_install, attempt_import
+    from meerschaum.utils.packages import pip_install
     from meerschaum.utils.misc import is_docker_available
 
     if action is None:
         action = []
 
-    is_stack_running = False
     try:
         success, msg = actions['stack'](
             ['ps', '-q'], _capture_output=True, debug=debug
@@ -69,14 +68,14 @@ def _upgrade_meerschaum(
         if msg.endswith('\n'):
             msg = msg[:-1]
         containers = msg.split('\n')
-    except Exception as e:
+    except Exception:
         containers = []
 
     if containers:
         if force:
             answer = True
         else:
-            answer = yes_no(f"Take down the stack?", default='y', yes=yes, noask=noask)
+            answer = yes_no("Take down the stack?", default='n', yes=yes, noask=noask)
 
         if answer:
             if debug:
@@ -94,52 +93,48 @@ def _upgrade_meerschaum(
     if debug:
         dprint('Upgrade meerschaum with dependencies: \"' + f'{dependencies}' + '\"')
     if not pip_install(install_name, venv=None, debug=debug):
-        return False, f"Failed to upgrade Meerschaum via pip."
+        return False, "Failed to upgrade Meerschaum via pip."
 
     if debug:
         dprint("Pulling new Docker images...")
     if is_docker_available():
         actions['stack'](['pull'], debug=debug)
 
-    return True, "Success"
+    return actions['reload'](debug=debug)
 
 
 class NoVenv:
     pass
 
+
 def _upgrade_packages(
-        action: Optional[List[str]] = None,
-        venv: Union[str, None, NoVenv] = NoVenv,
-        yes: bool = False,
-        force: bool = False,
-        noask: bool = False,
-        debug: bool = False,
-        **kw: Any
-    ) -> SuccessTuple:
+    action: Optional[List[str]] = None,
+    venv: Union[str, None, NoVenv] = NoVenv,
+    yes: bool = False,
+    force: bool = False,
+    noask: bool = False,
+    debug: bool = False,
+    **kw: Any
+) -> SuccessTuple:
     """
     Upgrade and install dependencies.
     If provided, upgrade only a dependency group, otherwise default to `full`.
-    
+
     Examples:
-        ```
         upgrade packages
-        ```
-    
-        ```
-        upgrade packages docs
-        ```
+        upgrade packages full
     """
-    from meerschaum.utils.packages import packages, pip_install
-    from meerschaum.utils.warnings import info, warn
+    from meerschaum.utils.packages import packages, pip_install, get_prerelease_dependencies
+    from meerschaum.utils.warnings import warn
     from meerschaum.utils.prompt import yes_no
     from meerschaum.utils.formatting import make_header, pprint
-    from meerschaum.utils.misc import print_options
+    from meerschaum.actions import actions
     if action is None:
         action = []
     if venv is NoVenv:
         venv = 'mrsm'
     if len(action) == 0:
-        group = 'full'
+        group = 'api'
     else:
         group = action[0]
 
@@ -147,7 +142,7 @@ def _upgrade_packages(
         invalid_msg = f"Invalid dependency group '{group}'."
         avail_msg = make_header("Available groups:")
         for k in packages:
-            avail_msg += "\n  - {k}"
+            avail_msg += f"\n  - {k}"
 
         warn(invalid_msg + "\n\n" + avail_msg, stack=False)
         return False, invalid_msg
@@ -159,24 +154,41 @@ def _upgrade_packages(
         f"(dependency group '{group}')?"
     )
     to_install = [install_name for import_name, install_name in packages[group].items()]
+    prereleases_to_install = get_prerelease_dependencies(to_install)
+    to_install = [
+        install_name
+        for install_name in to_install
+        if (install_name not in prereleases_to_install) or group == '_internal'
+    ]
 
-    success, msg = False, f"Nothing installed."
+    success, msg = False, "Nothing installed."
     if force or yes_no(question, noask=noask, yes=yes):
         success = pip_install(*to_install, debug=debug)
+        if success and prereleases_to_install:
+            success = pip_install(*prereleases_to_install, debug=debug)
         msg = (
             f"Successfully installed {len(packages[group])} packages." if success
             else f"Failed to install packages in dependency group '{group}'."
         )
+
+    if not success:
+        return success, msg
+
+    reload_success, reload_msg = actions['reload'](debug=debug)
+    if not reload_success:
+        return reload_success, reload_msg
+
     return success, msg
 
+
 def _upgrade_plugins(
-        action: Optional[List[str]] = None,
-        yes: bool = False,
-        force: bool = False,
-        noask: bool = False,
-        debug: bool = False,
-        **kw: Any
-    ) -> SuccessTuple:
+    action: Optional[List[str]] = None,
+    yes: bool = False,
+    force: bool = False,
+    noask: bool = False,
+    debug: bool = False,
+    **kw: Any
+) -> SuccessTuple:
     """
     Upgrade all installed plugins to the latest versions.
     If no plugins are specified, attempt to upgrade all,
@@ -184,12 +196,8 @@ def _upgrade_plugins(
     
     Examples:
     
-    ```
     upgrade plugins
-    ```
-    ```
-    upgrade plugins testing
-    ```
+    upgrade plugins noaa
     """
     from meerschaum.actions import actions
     from meerschaum.plugins import get_plugins_names
@@ -202,15 +210,24 @@ def _upgrade_plugins(
     to_install = get_plugins_names() if len(action) == 0 else action
     if len(to_install) == 0:
         return False, "No plugins to upgrade."
+
     print_options(to_install, header="Plugins to Upgrade:")
     if force or yes_no(f"Upgrade {len(to_install)} plugins?", yes=yes, noask=noask):
-        return actions['install'](
-            action=['plugins'] + to_install, debug=debug, force=force, noask=noask, yes=yes, **kw
+        install_success, install_msg = actions['install'](
+            action=['plugins'] + to_install,
+            debug=debug,
+            force=force,
+            noask=noask,
+            yes=yes,
+            **kw
         )
+
+        return install_success, install_msg
+
     return False, "No plugins upgraded."
 
 ### NOTE: This must be the final statement of the module.
 ###       Any subactions added below these lines will not
 ###       be added to the `help` docstring.
-from meerschaum.utils.misc import choices_docstring as _choices_docstring
+from meerschaum.actions import choices_docstring as _choices_docstring
 upgrade.__doc__ += _choices_docstring('upgrade')
