@@ -18,7 +18,9 @@ default_formatting_config = {
         'hand'             : '👋',
         'error'            : '🛑',
         'failure'          : '💢',
+        'failure_calm'     : '❌',
         'success'          : '🎉',
+        'success_calm'     : '✅',
         'warning'          : '🔔',
         'info'             : '💬',
         'debug'            : '🐞',
@@ -26,10 +28,17 @@ default_formatting_config = {
         'connector'        : '🔌',
         'metric'           : '📊',
         'location'         : '📍',
+        'locked'           : '🔒',
+        'unlocked'         : '🔓',
         'key'              : '🔑',
         'idea'             : '💡',
         'connected'        : '🟢',
         'disconnected'     : '🔴',
+        'running'          : '🟢',
+        'paused'           : '🟡',
+        'stopped'          : '🔴',
+        'tag'              : '🔖',
+        'announcement'     : '📢',
     },
     'pipes'                : {
         'unicode'          : {
@@ -38,6 +47,7 @@ default_formatting_config = {
                 'metric'   : 'MRSM{formatting:emoji:metric} ',
                 'location' : 'MRSM{formatting:emoji:location} ',
                 'key'      : 'MRSM{formatting:emoji:key} ',
+                'tag'      : 'MRSM{formatting:emoji:tag} ',
             },
         },
         'ascii'            : {
@@ -46,6 +56,7 @@ default_formatting_config = {
                 'metric'   : '',
                 'location' : '',
                 'key'      : '',
+                'tag'      : '',
             },
         },
         'ansi'             : {
@@ -56,6 +67,7 @@ default_formatting_config = {
                 'key'      : '',
                 'guide'    : 'dim',
                 'none'     : 'black on magenta',
+                'tags'     : 'bold yellow underline',
             },
         },
         '__repr__'         : {
@@ -110,6 +122,32 @@ default_formatting_config = {
             },
         },
     },
+    'success_calm'         : {
+        'unicode'          : {
+            'icon'         : 'MRSM{formatting:emoji:success_calm}',
+        },
+        'ascii'            : {
+            'icon'         : '+',
+        },
+        'ansi'             : {
+            'rich'         : {
+                'style'    : 'pale_green3',
+            },
+        },
+    },
+    'failure_calm'         : {
+        'unicode'          : {
+            'icon'         : 'MRSM{formatting:emoji:failure_calm}',
+        },
+        'ascii'            : {
+            'icon'         : 'x',
+        },
+        'ansi'             : {
+            'rich'         : {
+                'style'    : 'indian red',
+            },
+        },
+    },
     'errors'               : {
         'unicode'          : {
             'icon'         : 'MRSM{formatting:emoji:error}',
@@ -188,4 +226,5 @@ default_formatting_config = {
             },
         },
     },
+
 }
