@@ -326,23 +326,22 @@ def sync_plugins_symlinks(debug: bool = False, warn: bool = True) -> None:
     """
     Update the plugins' internal symlinks. 
     """
+    from meerschaum.utils.warnings import error, warn as _warn, dprint
     global _synced_symlinks
     with _locks['_synced_symlinks']:
         if _synced_symlinks:
+            if debug:
+                dprint("Skip syncing symlinks...")
             return
 
     import os
     import pathlib
     import time
     from collections import defaultdict
-    import importlib.util
     from meerschaum.utils.misc import flatten_list, make_symlink, is_symlink
-    from meerschaum.utils.warnings import error, warn as _warn
     from meerschaum._internal.static import STATIC_CONFIG
-    from meerschaum.utils.venv import Venv, activate_venv, deactivate_venv, is_venv_active
     from meerschaum.config._paths import (
         PLUGINS_RESOURCES_PATH,
-        PLUGINS_ARCHIVES_RESOURCES_PATH,
         PLUGINS_INIT_PATH,
         PLUGINS_DIR_PATHS,
         PLUGINS_INTERNAL_LOCK_PATH,
@@ -574,7 +573,7 @@ def import_plugins(
             for plugin_name in flatten_list(plugins_to_import):
                 plugin = Plugin(plugin_name)
                 try:
-                    with Venv(plugin):
+                    with Venv(plugin, init_if_not_exists=False):
                         imported_plugins.append(
                             importlib.import_module(
                                 f'{PLUGINS_RESOURCES_PATH.stem}.{plugin_name}'
@@ -700,7 +699,11 @@ def load_plugins(
     Import Meerschaum plugins and update the actions dictionary.
     """
     global _loaded_plugins
+    from meerschaum.utils.warnings import dprint
+
     if skip_if_loaded and _loaded_plugins:
+        if debug:
+            dprint("Skip loading plugins...")
         return
 
     from inspect import isfunction, getmembers
@@ -714,6 +717,7 @@ def load_plugins(
         recursive = True,
         modules_venvs = True
     )
+
     ### I'm appending here to keep from redefining the modules list.
     new_modules = (
         [
@@ -784,7 +788,7 @@ def unload_plugins(
     """
     Unload the specified plugins from memory.
     """
-    global _loaded_plugins
+    global _loaded_plugins, _synced_symlinks
     import sys
     from meerschaum.config.paths import PLUGINS_RESOURCES_PATH, PLUGINS_INJECTED_RESOURCES_PATH
     from meerschaum.connectors import unload_plugin_connectors
@@ -792,8 +796,10 @@ def unload_plugins(
         from meerschaum.utils.warnings import dprint
 
     _loaded_plugins = False
+    _synced_symlinks = False
 
-    plugins = plugins or get_plugins_names()
+    all_plugins = get_plugins_names()
+    plugins = plugins or all_plugins
     if debug:
         dprint(f"Unloading plugins: {plugins}")
 
@@ -803,11 +809,26 @@ def unload_plugins(
     module_prefix = f"{PLUGINS_RESOURCES_PATH.stem}."
     loaded_modules = [mod_name for mod_name in sys.modules if mod_name.startswith(module_prefix)]
 
+    root_plugins_mod = (
+        sys.modules.get(PLUGINS_RESOURCES_PATH.stem, None)
+        if sorted(plugins) != sorted(all_plugins)
+        else sys.modules.pop(PLUGINS_RESOURCES_PATH, None)
+    )
+
     for plugin_name in plugins:
         for mod_name in loaded_modules:
-            if mod_name[len(PLUGINS_RESOURCES_PATH.stem):].startswith(plugin_name):
+            if (
+                mod_name[len(PLUGINS_RESOURCES_PATH.stem):].startswith(plugin_name + '.')
+                or mod_name[len(PLUGINS_RESOURCES_PATH.stem):] == plugin_name
+            ):
                 _ = sys.modules.pop(mod_name, None)
 
+        if root_plugins_mod is not None and plugin_name in root_plugins_mod.__dict__:
+            try:
+                delattr(root_plugins_mod, plugin_name)
+            except Exception:
+                pass
+
         ### Unload sync hooks.
         _ = _pre_sync_hooks.pop(plugin_name, None)
         _ = _post_sync_hooks.pop(plugin_name, None)
@@ -890,7 +911,9 @@ def get_plugins(*to_load, try_import: bool = True) -> Union[Tuple[Plugin], Plugi
                 (
                     name if (PLUGINS_RESOURCES_PATH / name).is_dir()
                     else name[:-3]
-                ) for name in os.listdir(PLUGINS_RESOURCES_PATH) if name != '__init__.py'
+                )
+                for name in os.listdir(PLUGINS_RESOURCES_PATH)
+                if name != '__init__.py'
             ]
         )
     ]
