@@ -7,36 +7,36 @@ Pretty printing wrapper
 """
 
 def pprint(
-        *args,
-        detect_password : bool = True,
-        nopretty : bool = False,
-        **kw
-    ):
+    *args,
+    detect_password: bool = True,
+    nopretty: bool = False,
+    **kw
+) -> None:
     """Pretty print an object according to the configured ANSI and UNICODE settings.
     If detect_password is True (default), search and replace passwords with '*' characters.
     Does not mutate objects.
-
-    Parameters
-    ----------
-    *args :
-        
-    detect_password : bool :
-         (Default value = True)
-    nopretty : bool :
-         (Default value = False)
-    **kw :
-        
-
-    Returns
-    -------
-
     """
+    import copy
+    import json
     from meerschaum.utils.packages import attempt_import, import_rich
-    from meerschaum.utils.formatting import ANSI, UNICODE, get_console
+    from meerschaum.utils.formatting import ANSI, get_console, print_tuple
     from meerschaum.utils.warnings import error
     from meerschaum.utils.misc import replace_password, dict_from_od, filter_keywords
     from collections import OrderedDict
-    import copy, json
+
+    if (
+        len(args) == 1
+        and
+        isinstance(args[0], tuple)
+        and
+        len(args[0]) == 2
+        and
+        isinstance(args[0][0], bool)
+        and
+        isinstance(args[0][1], str)
+    ):
+        return print_tuple(args[0], **filter_keywords(print_tuple, **kw))
+
     modify = True
     rich_pprint = None
     if ANSI and not nopretty:
@@ -53,7 +53,7 @@ def pprint(
         pprintpp = attempt_import('pprintpp', warn=False)
         try:
             _pprint = pprintpp.pprint
-        except Exception as e:
+        except Exception :
             import pprint as _pprint_module
             _pprint = _pprint_module.pprint
 
@@ -63,9 +63,10 @@ def pprint(
 
     try:
         args_copy = copy.deepcopy(args)
-    except Exception as e:
+    except Exception:
         args_copy = args
         modify = False
+
     _args = []
     for a in args:
         c = a
@@ -86,12 +87,12 @@ def pprint(
                 try:
                     c = json.dumps(c)
                     is_json = True
-                except Exception as e:
+                except Exception:
                     is_json = False
                 if not is_json:
                     try:
                         c = str(c)
-                    except Exception as e:
+                    except Exception:
                         pass
             _args.append(c)
 
