@@ -8,6 +8,9 @@ Manage virtual environments.
 
 from __future__ import annotations
 
+import sys
+import pathlib
+
 from meerschaum.utils.typing import Optional, Union, Dict, List, Tuple
 from meerschaum.utils.threading import RLock, get_ident
 
@@ -15,7 +18,7 @@ __all__ = sorted([
     'activate_venv', 'deactivate_venv', 'init_venv',
     'inside_venv', 'is_venv_active', 'venv_exec',
     'venv_executable', 'venv_exists', 'venv_target_path',
-    'Venv', 'get_venvs', 'verify_venv',
+    'Venv', 'get_venvs', 'verify_venv', 'get_module_venv',
 ])
 __pdoc__ = {'Venv': True}
 
@@ -29,15 +32,17 @@ active_venvs = set()
 active_venvs_counts: Dict[str, int] = {}
 active_venvs_order: List[Optional[str]] = []
 threads_active_venvs: Dict[int, 'set[str]'] = {}
+CREATE_NEW_PROCESS_GROUP = 0x00000200
 
 
 def activate_venv(
-        venv: Optional[str] = 'mrsm',
-        color: bool = True,
-        force: bool = False,
-        debug: bool = False,
-        **kw
-    ) -> bool:
+    venv: Optional[str] = 'mrsm',
+    color: bool = True,
+    force: bool = False,
+    init_if_not_exists: bool = True,
+    debug: bool = False,
+    **kw
+) -> bool:
     """
     Create a virtual environment (if it doesn't exist) and add it to `sys.path` if necessary.
 
@@ -49,6 +54,9 @@ def activate_venv(
     color: bool, default True
         If `True`, include color in debug text.
 
+    init_if_not_exists: bool, default True
+        If `True`, create the virtual environment if it does not exist.
+
     force: bool, default False
         If `True`, do not exit early even if the venv is currently active.
 
@@ -64,12 +72,12 @@ def activate_venv(
     if active_venvs_order and active_venvs_order[0] == venv:
         if not force:
             return True
-    import sys, platform, os
-    from meerschaum.config._paths import VIRTENV_RESOURCES_PATH
-    if debug:
-        from meerschaum.utils.debug import dprint
-    if venv is not None:
+
+    import sys
+    import os
+    if venv is not None and init_if_not_exists:
         init_venv(venv=venv, debug=debug)
+
     with LOCKS['active_venvs']:
         if thread_id not in threads_active_venvs:
             threads_active_venvs[thread_id] = {}
@@ -79,13 +87,27 @@ def activate_venv(
         else:
             threads_active_venvs[thread_id][venv] += 1
 
-        target = str(venv_target_path(venv, debug=debug))
+        target_path = venv_target_path(venv, debug=debug, allow_nonexistent=True)
+        if not target_path.exists() and venv is not None and init_if_not_exists:
+            init_venv(venv=venv, force=True, debug=debug)
+
+        if not target_path.exists():
+            if init_if_not_exists:
+                return False
+            raise EnvironmentError(f"Could not activate virtual environment '{venv}'.")
+
+        target = target_path.as_posix()
+
         if venv in active_venvs_order:
-            sys.path.remove(target)
+            try:
+                sys.path.remove(target)
+            except Exception:
+                pass
             try:
                 active_venvs_order.remove(venv)
-            except Exception as e:
+            except Exception:
                 pass
+
         if venv is not None:
             sys.path.insert(0, target)
         else:
@@ -95,20 +117,20 @@ def activate_venv(
                 sys.path.insert(0, target)
         try:
             active_venvs_order.insert(0, venv)
-        except Exception as e:
+        except Exception:
             pass
 
     return True
 
 
 def deactivate_venv(
-        venv: str = 'mrsm',
-        color: bool = True,
-        debug: bool = False,
-        previously_active_venvs: Union['set[str]', List[str], None] = None,
-        force: bool = False,
-        **kw
-    ) -> bool:
+    venv: str = 'mrsm',
+    color: bool = True,
+    debug: bool = False,
+    previously_active_venvs: Union['set[str]', List[str], None] = None,
+    force: bool = False,
+    **kw
+) -> bool:
     """
     Remove a virtual environment from `sys.path` (if it's been activated).
 
@@ -171,23 +193,26 @@ def deactivate_venv(
     if sys.path is None:
         return False
 
-    target = str(venv_target_path(venv, allow_nonexistent=force, debug=debug))
+    target = venv_target_path(venv, allow_nonexistent=True, debug=debug).as_posix()
     with LOCKS['sys.path']:
         if target in sys.path:
-            sys.path.remove(target)
+            try:
+                sys.path.remove(target)
+            except Exception:
+                pass
             try:
                 active_venvs_order.remove(venv)
-            except Exception as e:
+            except Exception:
                 pass
 
     return True
 
 
 def is_venv_active(
-        venv: str = 'mrsm',
-        color : bool = True,
-        debug: bool = False
-    ) -> bool:
+    venv: str = 'mrsm',
+    color : bool = True,
+    debug: bool = False
+) -> bool:
     """
     Check if a virtual environment is active.
 
@@ -212,17 +237,21 @@ def is_venv_active(
 
 verified_venvs = set()
 def verify_venv(
-        venv: str,
-        debug: bool = False,
-    ) -> None:
+    venv: str,
+    debug: bool = False,
+) -> None:
     """
     Verify that the virtual environment matches the expected state.
     """
-    import pathlib, platform, os, shutil, subprocess, sys
+    import platform
+    import os
+    import shutil
+    import sys
     from meerschaum.config._paths import VIRTENV_RESOURCES_PATH
     from meerschaum.utils.process import run_process
     from meerschaum.utils.misc import make_symlink, is_symlink
     from meerschaum.utils.warnings import warn
+
     venv_path = VIRTENV_RESOURCES_PATH / venv
     bin_path = venv_path / (
         'bin' if platform.system() != 'Windows' else "Scripts"
@@ -244,7 +273,7 @@ def verify_venv(
                     print(f"Unable to remove symlink {current_python_in_venv_path}:\n{e}")
             try:
                 make_symlink(current_python_in_sys_path, current_python_in_venv_path)
-            except Exception as e:
+            except Exception:
                 print(
                     f"Unable to create symlink {current_python_in_venv_path} "
                     + f"to {current_python_in_sys_path}."
@@ -263,8 +292,8 @@ def verify_venv(
                 print(f"Getting python version for {python_path}")
             proc = run_process(
                 [str(python_path), '-V'],
-                as_proc = True,
-                capture_output = True,
+                as_proc=True,
+                capture_output=True,
             )
             stdout, stderr = proc.communicate(timeout=1.0)
         except Exception as e:
@@ -285,8 +314,11 @@ def verify_venv(
         version = get_python_version(python_path)
         if version is None:
             continue
-        major_version = version.split('.', maxsplit=1)[0]
-        minor_version = version.split('.', maxsplit=2)[1]
+        try:
+            major_version = version.split('.', maxsplit=1)[0]
+            minor_version = version.split('.', maxsplit=2)[1]
+        except IndexError:
+            return
         python_versioned_name = (
             'python' + major_version + '.' + minor_version
             + ('' if platform.system() != 'Windows' else '.exe')
@@ -298,7 +330,7 @@ def verify_venv(
             if not real_path.exists():
                 try:
                     python_path.unlink()
-                except Exception as e:
+                except Exception:
                     pass
                 init_venv(venv, verify=False, force=True, debug=debug)
                 if not python_path.exists():
@@ -309,7 +341,7 @@ def verify_venv(
 
             try:
                 python_path.unlink()
-            except Exception as e:
+            except Exception:
                 pass
             success, msg = make_symlink(real_path, python_path)
             if not success:
@@ -327,11 +359,11 @@ def verify_venv(
 
 tried_virtualenv = False
 def init_venv(
-        venv: str = 'mrsm',
-        verify: bool = True,
-        force: bool = False,
-        debug: bool = False,
-    ) -> bool:
+    venv: str = 'mrsm',
+    verify: bool = True,
+    force: bool = False,
+    debug: bool = False,
+) -> bool:
     """
     Initialize the virtual environment.
 
@@ -358,29 +390,114 @@ def init_venv(
             verified_venvs.add(venv)
         return True
 
-    import sys, platform, os, pathlib, shutil
-    from meerschaum.config._paths import VIRTENV_RESOURCES_PATH
-    from meerschaum.utils.packages import run_python_package, attempt_import
+    import io
+    from contextlib import redirect_stdout
+    import sys
+    import platform
+    import os
+    import shutil
+    import time
+
+    from meerschaum._internal.static import STATIC_CONFIG
+    from meerschaum.config._paths import (
+        VIRTENV_RESOURCES_PATH,
+        VENVS_CACHE_RESOURCES_PATH,
+    )
+    from meerschaum.utils.packages import is_uv_enabled
+
+    venv_path = VIRTENV_RESOURCES_PATH / venv
+    vtp = venv_target_path(venv=venv, allow_nonexistent=True, debug=debug)
+    docker_home_venv_path = pathlib.Path('/home/meerschaum/venvs/mrsm')
+    lock_path = VENVS_CACHE_RESOURCES_PATH / (venv + '.lock')
+    work_dir_env_var = STATIC_CONFIG['environment']['work_dir']
+
+    def update_lock(active: bool):
+        try:
+            if not active:
+                if debug:
+                    print(f"Releasing lock: '{lock_path}'")
+                lock_path.unlink()
+            else:
+                if debug:
+                    print(f"Acquiring lock: '{lock_path}'")
+                lock_path.touch()
+        except Exception:
+            pass
+
+    def wait_for_lock():
+        if platform.system() == 'Windows':
+            return
+        max_lock_seconds = 30.0
+        sleep_message_seconds = 5.0
+        step_sleep_seconds = 0.1
+        init_venv_check_start = time.perf_counter()
+        last_print = init_venv_check_start
+        while ((time.perf_counter() - init_venv_check_start) < max_lock_seconds):
+            if not lock_path.exists():
+                break
+
+            now = time.perf_counter()
+            if debug or (now - last_print) > sleep_message_seconds:
+                print(f"Lock exists for venv '{venv}', sleeping...")
+                last_print = now
+            time.sleep(step_sleep_seconds)
+        update_lock(False)
+
+    if (
+        not force
+        and venv == 'mrsm'
+        and os.environ.get(work_dir_env_var, None) is not None
+        and docker_home_venv_path.exists()
+    ):
+        wait_for_lock()
+        shutil.move(docker_home_venv_path, venv_path)
+        if verify:
+            verify_venv(venv, debug=debug)
+            verified_venvs.add(venv)
+        return True
+
+    from meerschaum.utils.packages import run_python_package, attempt_import, _get_pip_os_env
     global tried_virtualenv
     try:
         import venv as _venv
+        uv = attempt_import('uv', venv=None, debug=debug) if is_uv_enabled() else None
         virtualenv = None
     except ImportError:
         _venv = None
+        uv = None
         virtualenv = None
-    
-    venv_path = VIRTENV_RESOURCES_PATH / venv
 
     _venv_success = False
-    if _venv is not None:
-        import io
-        from contextlib import redirect_stdout
+    temp_vtp = VENVS_CACHE_RESOURCES_PATH / str(venv)
+    ### NOTE: Disable site-packages movement for now.
+    rename_vtp = False and vtp.exists() and not temp_vtp.exists()
+
+    wait_for_lock()
+    update_lock(True)
+
+    if rename_vtp:
+        if debug:
+            print(f"Moving '{vtp}' to '{temp_vtp}'...")
+        shutil.move(vtp, temp_vtp)
+
+    if uv is not None:
+        _venv_success = run_python_package(
+            'uv',
+            ['venv', venv_path.as_posix(), '-q', '--no-project', '--allow-existing', '--seed'],
+            venv=None,
+            env=_get_pip_os_env(),
+            debug=debug,
+        ) == 0
+
+    if _venv is not None and not _venv_success:
         f = io.StringIO()
         with redirect_stdout(f):
             _venv_success = run_python_package(
                 'venv',
-                [str(venv_path)] + (
-                    ['--symlinks'] if platform.system() != 'Windows' else []
+                [venv_path.as_posix()] + (
+                    ['--symlinks']
+                    if platform.system() != 'Windows'
+                    else []
                 ),
                 venv=None, debug=debug
             ) == 0
@@ -390,14 +507,25 @@ def init_venv(
             _venv = None
     if not _venv_success:
         virtualenv = attempt_import(
-            'virtualenv', venv=None, lazy=False, install=(not tried_virtualenv), warn=False,
-            check_update=False, color=False, debug=debug,
+            'virtualenv',
+            venv=None,
+            lazy=False,
+            install=(not tried_virtualenv),
+            warn=False,
+            check_update=False,
+            color=False,
+            debug=debug,
         )
         if virtualenv is None:
             print(
                 "Failed to import `venv` or `virtualenv`! "
                 + "Please install `virtualenv` via pip then restart Meerschaum."
             )
+            if rename_vtp and temp_vtp.exists():
+                if debug:
+                    print(f"Moving '{temp_vtp}' back to '{vtp}'...")
+                shutil.move(temp_vtp, vtp)
+            update_lock(False)
             return False
 
         tried_virtualenv = True
@@ -415,10 +543,10 @@ def init_venv(
             if bin_path.exists():
                 try:
                     shutil.rmtree(bin_path)
-                except Exception as e:
+                except Exception:
                     import traceback
                     traceback.print_exc()
-            virtualenv.cli_run([str(venv_path)])
+            virtualenv.cli_run([venv_path.as_posix()])
             if dist_packages_path.exists():
                 vtp.mkdir(exist_ok=True, parents=True)
                 for file_path in dist_packages_path.glob('*'):
@@ -431,13 +559,23 @@ def init_venv(
                 #  shutil.move(local_bin_path, bin_path)
                 shutil.rmtree(local_bin_path)
 
-        except Exception as e:
+        except Exception:
             import traceback
             traceback.print_exc()
+            if rename_vtp and temp_vtp.exists():
+                shutil.move(temp_vtp, vtp)
+            update_lock(False)
             return False
     if verify:
         verify_venv(venv, debug=debug)
         verified_venvs.add(venv)
+
+    if rename_vtp and temp_vtp.exists():
+        if debug:
+            print(f"Cleanup: move '{temp_vtp}' back to '{vtp}'.")
+        shutil.move(temp_vtp, vtp)
+
+    update_lock(False)
     return True
 
 
@@ -446,7 +584,8 @@ def venv_executable(venv: Optional[str] = 'mrsm') -> str:
     The Python interpreter executable for a given virtual environment.
     """
     from meerschaum.config._paths import VIRTENV_RESOURCES_PATH
-    import sys, platform, os
+    import sys
+    import platform
     return (
         sys.executable if venv is None
         else str(
@@ -466,13 +605,14 @@ def venv_executable(venv: Optional[str] = 'mrsm') -> str:
 
 
 def venv_exec(
-        code: str,
-        venv: Optional[str] = 'mrsm',
-        with_extras: bool = False,
-        as_proc: bool = False,
-        capture_output: bool = True,
-        debug: bool = False,
-    ) -> Union[bool, Tuple[int, bytes, bytes]]:
+    code: str,
+    venv: Optional[str] = 'mrsm',
+    env: Optional[Dict[str, str]] = None,
+    with_extras: bool = False,
+    as_proc: bool = False,
+    capture_output: bool = True,
+    debug: bool = False,
+) -> Union[bool, Tuple[int, bytes, bytes], 'subprocess.Popen']:
     """
     Execute Python code in a subprocess via a virtual environment's interpeter.
     Return `True` if the code successfully executes, `False` on failure.
@@ -486,6 +626,10 @@ def venv_exec(
         The virtual environment to use to get the path for the Python executable.
         If `venv` is `None`, use the default `sys.executable` path.
 
+    env: Optional[Dict[str, str]], default None
+        Optionally specify the environment variables for the subprocess.
+        Defaults to `os.environ`.
+
     with_extras: bool, default False
         If `True`, return a tuple of the exit code, stdout bytes, and stderr bytes.
 
@@ -501,22 +645,38 @@ def venv_exec(
     """
     import os
     import subprocess
+    import platform
     from meerschaum.utils.debug import dprint
+    from meerschaum.utils.process import _child_processes
+
     executable = venv_executable(venv=venv)
     cmd_list = [executable, '-c', code]
+    if env is None:
+        env = os.environ
     if debug:
         dprint(str(cmd_list))
     if not with_extras and not as_proc:
-        return subprocess.call(cmd_list, env=os.environ) == 0
+        return subprocess.call(cmd_list, env=env) == 0
 
     stdout, stderr = (None, None) if not capture_output else (subprocess.PIPE, subprocess.PIPE)
+    group_kwargs = (
+        {
+            'preexec_fn': os.setsid,
+        } if platform.system() != 'Windows'
+        else {
+            'creationflags': CREATE_NEW_PROCESS_GROUP,
+        }
+    )
     process = subprocess.Popen(
         cmd_list,
-        stdout = stdout,
-        stderr = stderr,
-        env = os.environ,
+        stdout=stdout,
+        stderr=stderr,
+        stdin=sys.stdin,
+        env=env,
+        **group_kwargs
     )
     if as_proc:
+        _child_processes.append(process)
         return process
     stdout, stderr = process.communicate()
     exit_code = process.returncode
@@ -532,10 +692,10 @@ def venv_exists(venv: Union[str, None], debug: bool = False) -> bool:
 
 
 def venv_target_path(
-        venv: Union[str, None],
-        allow_nonexistent: bool = False,
-        debug: bool = False,
-    ) -> 'pathlib.Path':
+    venv: Union[str, None],
+    allow_nonexistent: bool = False,
+    debug: bool = False,
+) -> pathlib.Path:
     """
     Return a virtual environment's site-package path.
 
@@ -552,9 +712,12 @@ def venv_target_path(
     The `pathlib.Path` object for the virtual environment's path.
 
     """
-    import os, sys, platform, pathlib, site
+    import os
+    import sys
+    import platform
+    import site
     from meerschaum.config._paths import VIRTENV_RESOURCES_PATH
-    from meerschaum.config.static import STATIC_CONFIG
+    from meerschaum._internal.static import STATIC_CONFIG
 
     ### Check sys.path for a user-writable site-packages directory.
     if venv is None:
@@ -578,23 +741,33 @@ def venv_target_path(
             return site_packages_path
 
         if not inside_venv():
-            site_path = pathlib.Path(site.getusersitepackages())
-            ### Allow for dist-level paths (running as root).
+            user_site_packages = site.getusersitepackages()
+            if user_site_packages is None:
+                raise EnvironmentError("Could not determine user site packages.")
+
+            site_path = pathlib.Path(user_site_packages)
             if not site_path.exists():
-                if platform.system() == 'Windows' or os.geteuid() == 0:
-                    for possible_dist in reversed(site.getsitepackages()):
-                        dist_path = pathlib.Path(possible_dist)
-                        if not dist_path.exists():
-                            continue
-                        return dist_path
-                    
-                    raise EnvironmentError("Could not determine the dist-packages directory.")
+
+                ### Windows does not have `os.geteuid()`.
+                if platform.system() == 'Windows' or os.geteuid() != 0:
+                    site_path.mkdir(parents=True, exist_ok=True)
+                    return site_path
+
+                ### Allow for dist-level paths (running as root).
+                for possible_dist in site.getsitepackages():
+                    dist_path = pathlib.Path(possible_dist)
+                    if not dist_path.exists():
+                        continue
+                    return dist_path
+                
+                raise EnvironmentError("Could not determine the dist-packages directory.")
 
             return site_path
 
     venv_root_path = (
         (VIRTENV_RESOURCES_PATH / venv)
-        if venv is not None else pathlib.Path(sys.prefix)
+        if venv is not None
+        else pathlib.Path(sys.prefix)
     )
     target_path = venv_root_path
 
@@ -612,12 +785,14 @@ def venv_target_path(
     python_folder = 'python' + str(sys.version_info.major) + '.' + str(sys.version_info.minor)
     if target_path.exists():
         target_path = (
-            (target_path / python_folder) if python_folder in os.listdir(target_path)
+            (target_path / python_folder)
+            if python_folder in os.listdir(target_path)
             else target_path
         )
     else:
         target_path = (
-            (target_path / python_folder) if platform.system() != 'Windows'
+            (target_path / python_folder)
+            if platform.system() != 'Windows'
             else target_path
         )
 
@@ -666,4 +841,27 @@ def get_venvs() -> List[str]:
     return venvs
 
 
+def get_module_venv(module) -> Union[str, None]:
+    """
+    Return the virtual environment where an imported module is installed.
+
+    Parameters
+    ----------
+    module: ModuleType
+        The imported module to inspect.
+
+    Returns
+    -------
+    The name of a venv or `None`.
+    """
+    from meerschaum.config.paths import VIRTENV_RESOURCES_PATH
+    module_path = pathlib.Path(module.__file__).resolve()
+    try:
+        rel_path = module_path.relative_to(VIRTENV_RESOURCES_PATH)
+    except ValueError:
+        return None
+
+    return rel_path.as_posix().split('/', maxsplit=1)[0]
+
+
 from meerschaum.utils.venv._Venv import Venv
